@@ -13,7 +13,7 @@
 """
 Defines all the model classes for the various parts of the API.
 """
-
+import itertools
 import logging
 import json
 import six
@@ -829,6 +829,15 @@ class SearchDsl(base.QueryableModel):
                      'attributes': AttributeDef,
                      'fullTextResults': FullTextResult}
 
+    def flatten_attrs(self):
+        """
+        If you are specifying attributes using a SELECT clause in DSL search,
+        you can use this function to flatten the attribute values.
+        :return: Python list of attributes values.
+        """
+        attributes = self._data.get('attributes').get("values") or list()
+        return list(itertools.chain.from_iterable(attributes))
+
 
 class SearchFulltextCollection(base.QueryableModelCollection):
     def load(self, response):
@@ -846,3 +855,79 @@ class SearchFulltext(base.QueryableModel):
     relationships = {'entities': Entity,
                      'attributes': AttributeDef,
                      'fullTextResults': FullTextResult}
+
+
+class SearchSavedCollection(base.QueryableModelCollection):
+    """We can also remove this part but just keeping same structure as above"""
+    def load(self, response):
+        for item in response:
+            model = self.model_class(self,
+                                     href=item.get('href'))
+            model.load(item)
+            self._models.append(model)
+
+
+class SearchSaved(base.QueryableModel):
+    collection_class = SearchSavedCollection
+    path = 'search/saved'
+    data_key = 'search_saved'
+    primary_key = 'guid'
+    fields = ('name', 'searchParameters', 'ownerName', 'searchType', 'uiParameters')
+
+    def inflate(self):
+        """Load the resource from the server, if not already loaded. Removed params check for searchParameters"""
+        if not self._is_inflated:
+            if self._is_inflating:
+                #  catch infinite recursion when attempting to inflate
+                #  an object that doesn't have enough data to inflate
+                msg = ("There is not enough data to inflate this object.  "
+                       "Need either an href: {} or a {}: {}")
+                msg = msg.format(self._href, self.primary_key, self._data.get(self.primary_key))
+                raise exceptions.ClientError(msg)
+
+            self._is_inflating = True
+
+            try:
+                params = {}
+                # To keep the method same as the original request. The default is GET
+                self.load(self.client.request(self.method, self.url, **params))
+            except Exception:
+                self.load(self._data)
+
+            self._is_inflated = True
+            self._is_inflating = False
+        return self
+
+    @events.evented
+    def create(self, data, **kwargs):
+        """This is override method.
+        """
+        self.client.post(self.url, data=data)
+        return self
+
+    @events.evented
+    def update(self, data, **kwargs):
+        """
+        This is override method
+        """
+        self.method = 'put'
+        self.load(self.client.put(self.parent.url, data=data))
+        return self
+
+
+class AdminMetricsCollection(base.QueryableModelCollection):
+    @property
+    def is_admin_api(self):
+        return True
+
+    def load(self, response):
+        model = self.model_class(self, href=self.url)
+        model.load(response)
+        self._models.append(model)
+
+
+class AdminMetrics(base.QueryableModel):
+    collection_class = AdminMetricsCollection
+    path = 'metrics'
+    data_key = 'data'
+    fields = ('general', 'tag', 'entity')
